#include "qglobalshortcut.h"
#include <QCoreApplication>

QMultiHash<quint32, QGlobalShortcut*> QGlobalShortcut::shortcuts_;
QGlobalShortcut::QGlobalShortcutEventFilter QGlobalShortcut::global_shortcut_event_;

QGlobalShortcut::QGlobalShortcut(QObject* parent)
    : QObject(parent)
{
    initialize();
}

QGlobalShortcut::QGlobalShortcut(const QKeySequence &keyseq, QObject *parent)
    : QObject(parent)
{
    initialize();
    setKey(keyseq);
}

void QGlobalShortcut::initialize() {
    static bool initialized = false;
    if (!initialized) {
        qApp->installNativeEventFilter(&global_shortcut_event_);
        initialized = true;
    }
}

QGlobalShortcut::~QGlobalShortcut() {

}

QKeySequence QGlobalShortcut::key() const
{
    return keyseq_;
}

bool QGlobalShortcut::setKey(const QKeySequence& keyseq)
{
    if (!keyseq_.isEmpty()) {
        unsetKey();
    }
    quint32 keyid = calcId(keyseq);
    if (shortcuts_.count(keyid) == 0) {
        quint32 keycode = toNativeKeycode(getKey(keyseq));
        quint32 mods = toNativeModifiers(getMods(keyseq));
#ifdef Q_OS_WIN
        if(!registerKey(keycode, mods, keyid)) {
            error(getErrorMsg());
            return false;
        }
#else
        if(!registerKey(keycode, mods, keyid)) {
            error("Hotkey already set");
            return false;
        }
#endif
    }
    this->keyseq_ = keyseq;
    shortcuts_.insert(keyid, this);
    return true;
}

bool QGlobalShortcut::unsetKey() {
    quint32 keyid = calcId(keyseq_);
    if (shortcuts_.remove(keyid, this) > 0) {
        if (shortcuts_.count(keyid) == 0) {
            quint32 keycode = toNativeKeycode(getKey(keyseq_));
            quint32 mods = toNativeModifiers(getMods(keyseq_));
#ifdef Q_OS_WIN
            if(!unregisterKey(keycode, mods, keyid)) {
                error(getErrorMsg());
                return false;
            }
#else
            if(!unregisterKey(keycode, mods, keyid)) {
                error("No hotkey set");
                return false;
            }
#endif
            keyseq_ = QKeySequence(); // clear
        }
    }
    return true;
}

bool QGlobalShortcut::activate(quint32 id) {
    if (shortcuts_.contains(id)) {
        for (QGlobalShortcut* s: shortcuts_.values(id)) {
            emit s->activated();
        }
        return true;
    }
    return false;
}

quint32 QGlobalShortcut::calcId(const QKeySequence& keyseq) {
    quint32 keycode = toNativeKeycode(getKey(keyseq));
    quint32 mods    = toNativeModifiers(getMods(keyseq));
    return calcId(keycode, mods);
}

Qt::Key QGlobalShortcut::getKey(const QKeySequence& keyseq) {
    if (keyseq.isEmpty()) {
        return Qt::Key(0);
    }
    return Qt::Key(keyseq[0] & ~Qt::KeyboardModifierMask);
}

Qt::KeyboardModifiers QGlobalShortcut::getMods(const QKeySequence& keyseq) {
    if (keyseq.isEmpty()) {
        return Qt::KeyboardModifiers(0);
    }
    return Qt::KeyboardModifiers(keyseq[0] & Qt::KeyboardModifierMask);
}
